@@ -1,5 +1,7 @@
+"""trident api"""
 from __future__ import absolute_import
-
+from __future__ import division
+from __future__ import print_function
 import sys
 from importlib import reload
 from sys import stderr
@@ -11,14 +13,11 @@ if sys.getdefaultencoding() != defaultencoding:
 
 
 
-__version__ = '0.5.2'
+__version__ = '0.5.7'
 stderr.write('trident {0}\n'.format(__version__))
-from .backend import *
-from trident import models
-from trident import misc
-from trident import callbacks
-from trident import data
 
+
+from trident.backend import *
 import threading
 
 
