@@ -1,3 +1,7 @@
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import print_function
+
 import hashlib
 import itertools
 import os
@@ -13,14 +17,14 @@ from typing import List, TypeVar, Iterable, Tuple, Union
 import numpy as np
 from skimage import color
 
-from .bbox_common import xywh_to_xyxy, xyxy_to_xywh
-from .image_common import gray_scale, image2array, mask2array, image_backend_adaptive, reverse_image_backend_adaptive, \
+from trident.data.bbox_common import xywh2xyxy, xyxy2xywh
+from trident.data.image_common import gray_scale, image2array, mask2array, image_backend_adaption, reverse_image_backend_adaption, \
     unnormalize, array2image, ExpectDataType, GetImageMode
-from .label_common import label_backend_adaptive
-from .mask_common import mask_backend_adaptive, color2label
-from .samplers import *
-from ..backend.common import DataSpec, PrintException, OrderedDict
-from ..backend.load_backend import get_backend
+from trident.data.label_common import label_backend_adaptive
+from trident.data.mask_common import mask_backend_adaptive, color2label
+from trident.data.samplers import *
+from trident.backend.common import DataSpec, PrintException, OrderedDict,Signature
+from trident.backend.load_backend import get_backend
 
 try:
     import Queue
@@ -28,10 +32,10 @@ except ImportError:
     import queue as Queue
 
 if get_backend() == 'pytorch':
-    from ..backend.pytorch_backend import to_numpy, to_tensor, ReplayBuffer
+    from trident.backend.pytorch_backend import to_numpy, to_tensor, ReplayBuffer
     import torch
 
-__all__ = ['Dataset', 'ImageDataset', 'MaskDataset', 'LabelDataset', 'BboxDataset', 'SsdBboxDataset', 'Iterator',
+__all__ = ['Dataset', 'ImageDataset', 'MaskDataset', 'LabelDataset', 'BboxDataset', 'MultipleDataset','MetricDataset', 'Iterator',
            'NumpyDataset', 'RandomNoiseDataset']
 
 T = TypeVar('T', int, float, str, np.ndarray)
@@ -56,6 +60,39 @@ class Dataset(List):
         return super().__len__()
 
 
+
+class MultipleDataset(List):
+    def __init__(self, datasets,symbol=None, expect_data_type=None, name=''):
+        super().__init__()
+        self.parameter = None
+        self.name = name
+        self.symbol = symbol
+        self.is_pair_process = False
+        self.expect_data_type = expect_data_type
+        for d in datasets:
+            self.__add__(d)
+
+    def __add__(self, other):
+        if super().__len__()==0 and isinstance(other,Dataset):
+            super().append(other)
+        elif isinstance(other,Dataset):
+            if len(super().__getitem__(0))==len(other):
+                super().append(other)
+            else:
+                raise ValueError('the dataset you add does not have same length with the existing dataset')
+
+    def add(self,other):
+        self.__add__(other)
+
+    def __getitem__(self, index: int):
+        results=[]
+        for i in range(super().__len__()):
+            results.append(super().__getitem__(i).__getitem__(index) )
+        return tuple(results)
+
+    def __len__(self):
+        return len(super().__getitem__(0))
+
 class ImageDataset(Dataset):
     def __init__(self, images=None, expect_data_type: ExpectDataType = ExpectDataType.rgb,
                  get_image_mode: GetImageMode = GetImageMode.processed, symbol=None, name=''):
@@ -101,7 +138,7 @@ class ImageDataset(Dataset):
             img = img.astype(self.dtype)
 
         if self.get_image_mode == GetImageMode.expect and self.is_pair_process == False:
-            return image_backend_adaptive(img)
+            return image_backend_adaption(img)
         elif self.get_image_mode == GetImageMode.processed and self.is_pair_process == False:
             return self.image_transform(img)
         elif self.is_pair_process == True:
@@ -111,14 +148,14 @@ class ImageDataset(Dataset):
 
     def image_transform(self, img_data):
         if len(self.image_transform_funcs) == 0:
-            return image_backend_adaptive(img_data)
+            return image_backend_adaption(img_data)
         if isinstance(img_data, np.ndarray):
             for fc in self.image_transform_funcs:
                 if not fc.__qualname__.startswith(
                         'random_') or 'crop' in fc.__qualname__ or 'rescale' in fc.__qualname__ or (
                         fc.__qualname__.startswith('random_') and random.randint(0, 10) % 2 == 0):
                     img_data = fc(img_data)
-            img_data = image_backend_adaptive(img_data)
+            img_data = image_backend_adaption(img_data)
 
             return img_data
         else:
@@ -127,7 +164,7 @@ class ImageDataset(Dataset):
     @property
     def reverse_image_transform_funcs(self):
         return_list = []
-        return_list.append(reverse_image_backend_adaptive)
+        return_list.append(reverse_image_backend_adaption)
         for i in range(len(self.image_transform_funcs)):
             fn = self.image_transform_funcs[-1 - i]
             if fn.__qualname__ == 'normalize.<locals>.img_op':
@@ -137,12 +174,132 @@ class ImageDataset(Dataset):
 
     def reverse_image_transform(self, img_data):
         if len(self.reverse_image_transform_funcs) == 0:
-            return reverse_image_backend_adaptive(img_data)
+            return reverse_image_backend_adaption(img_data)
         if isinstance(img_data, np.ndarray):
             # if img_data.ndim>=2:
             for fc in self.reverse_image_transform_funcs:
                 img_data = fc(img_data)
-            img_data = reverse_image_backend_adaptive(img_data)
+            img_data = reverse_image_backend_adaption(img_data)
+
+            return img_data
+        else:
+            return img_data
+
+class MetricDataset(Dataset):
+    def __init__(self, identities=None, expect_data_type: ExpectDataType = ExpectDataType.array_data,
+                 get_image_mode: GetImageMode = GetImageMode.processed, symbol=None, name=''):
+        super().__init__(symbol=symbol, expect_data_type=expect_data_type, name=name)
+        _defines={'triplet loss':('anchor','positive','negative'), }
+        self.__add__(identities)
+        self.dtype = np.float32
+        self.get_image_mode = get_image_mode
+        self.image_transform_funcs = []
+        self.is_pair_process = False
+
+    def __getitem__(self, index: int):
+        anchor=None
+        positive=None
+        negative=None
+        imgs = super().__getitem__(index).value
+        index_internal=  random.choice(range(len(imgs)))
+        anchor=imgs[index_internal]
+        if len(imgs)>1:
+            available_items=range(len(imgs))
+            list(available_items).remove(index_internal)
+            index_internal2=random.choice(available_items)
+            positive=imgs[index_internal2]
+            available_items = range(len(super()))
+            list(available_items).remove(index)
+            index2=random.choice(available_items)
+            negative=random.choice(super()[index2].value)
+
+        if isinstance(anchor, str):
+            anchor = image2array(anchor)
+            positive = image2array(positive)
+            negative = image2array(negative)
+
+
+        if not isinstance(anchor, np.ndarray):
+            raise ValueError('image data should be ndarray')
+        elif isinstance(anchor, np.ndarray) and anchor.ndim not in [2, 3]:
+            raise ValueError('image data dimension  should be 2 or 3, but get {0}'.format(anchor.ndim))
+        elif self.expect_data_type == ExpectDataType.gray:
+            anchor = color.rgb2gray(anchor).astype(self.dtype)
+            positive = color.rgb2gray(positive).astype(self.dtype)
+            negative = color.rgb2gray(negative).astype(self.dtype)
+        elif self.expect_data_type == ExpectDataType.rgb and anchor.ndim == 2:
+            anchor = np.repeat(np.expand_dims(anchor, -1), 3, -1).astype(self.dtype)
+            positive = np.repeat(np.expand_dims(positive, -1), 3, -1).astype(self.dtype)
+            negative = np.repeat(np.expand_dims(negative, -1), 3, -1).astype(self.dtype)
+        elif self.expect_data_type == ExpectDataType.rgb and anchor.ndim == 3:
+            anchor = anchor[:, :, :3].astype(self.dtype)
+            positive = positive[:, :, :3].astype(self.dtype)
+            negative = negative[:, :, :3].astype(self.dtype)
+        elif self.expect_data_type == ExpectDataType.rgba:
+            if anchor.ndim == 2:
+                anchor = np.repeat(np.expand_dims(anchor, -1), 3, -1)
+                positive = np.repeat(np.expand_dims(positive, -1), 3, -1)
+                negative = np.repeat(np.expand_dims(negative, -1), 3, -1)
+            if anchor.shape[2] == 3:
+                anchor = np.concatenate([anchor, np.ones((anchor.shape[0], anchor.shape[1], 1)) * 255], axis=-1)
+                positive = np.concatenate([positive, np.ones((positive.shape[0], positive.shape[1], 1)) * 255], axis=-1)
+                negative = np.concatenate([negative, np.ones((negative.shape[0], negative.shape[1], 1)) * 255], axis=-1)
+            anchor = anchor.astype(self.dtype)
+        elif self.expect_data_type == ExpectDataType.multi_channel:
+            anchor = anchor.astype(self.dtype)
+            negative = negative.astype(self.dtype)
+            positive= positive.astype(self.dtype)
+
+        if self.get_image_mode == GetImageMode.expect and self.is_pair_process == False:
+            anchor=image_backend_adaption(anchor)
+            negative = image_backend_adaption(negative)
+            positive = image_backend_adaption(positive)
+            return  anchor,positive,positive, negative
+        elif self.get_image_mode == GetImageMode.processed and self.is_pair_process == False:
+            anchor = self.image_transform(anchor)
+            negative = self.image_transform(negative)
+            positive = self.image_transform(positive)
+
+            return anchor,positive, negative
+        elif self.is_pair_process == True:
+            return  anchor,positive, negative
+
+        return None
+
+    def image_transform(self, img_data):
+        if len(self.image_transform_funcs) == 0:
+            return image_backend_adaption(img_data)
+        if isinstance(img_data, np.ndarray):
+            for fc in self.image_transform_funcs:
+                if not fc.__qualname__.startswith(
+                        'random_') or 'crop' in fc.__qualname__ or 'rescale' in fc.__qualname__ or (
+                        fc.__qualname__.startswith('random_') and random.randint(0, 10) % 2 == 0):
+                    img_data = fc(img_data)
+            img_data = image_backend_adaption(img_data)
+
+            return img_data
+        else:
+            return img_data
+
+    @property
+    def reverse_image_transform_funcs(self):
+        return_list = []
+        return_list.append(reverse_image_backend_adaption)
+        for i in range(len(self.image_transform_funcs)):
+            fn = self.image_transform_funcs[-1 - i]
+            if fn.__qualname__ == 'normalize.<locals>.img_op':
+                return_list.append(unnormalize(fn.mean, fn.std))
+        return_list.append(array2image)
+        return return_list
+
+    def reverse_image_transform(self, img_data):
+        if len(self.reverse_image_transform_funcs) == 0:
+            return reverse_image_backend_adaption(img_data)
+        if isinstance(img_data, np.ndarray):
+            # if img_data.ndim>=2:
+            for fc in self.reverse_image_transform_funcs:
+                img_data = fc(img_data)
+            img_data = reverse_image_backend_adaption(img_data)
 
             return img_data
         else:
@@ -235,7 +392,7 @@ class MaskDataset(Dataset):
     @property
     def reverse_image_transform_funcs(self):
         return_list = []
-        return_list.append(reverse_image_backend_adaptive)
+        return_list.append(reverse_image_backend_adaption)
         for i in range(len(self.image_transform_funcs)):
             fn = self.image_transform_funcs[-1 - i]
             if fn.__qualname__ == 'normalize.<locals>.img_op':
@@ -245,12 +402,12 @@ class MaskDataset(Dataset):
 
     def reverse_image_transform(self, img_data):
         if len(self.reverse_image_transform_funcs) == 0:
-            return reverse_image_backend_adaptive(img_data)
+            return reverse_image_backend_adaption(img_data)
         if isinstance(img_data, np.ndarray):
             # if img_data.ndim>=2:
             for fc in self.reverse_image_transform_funcs:
                 img_data = fc(img_data)
-            img_data = reverse_image_backend_adaptive(img_data)
+            img_data = reverse_image_backend_adaption(img_data)
 
             return img_data
         else:
@@ -342,133 +499,29 @@ class BboxDataset(Dataset):
         return bbox
 
 
-class SsdBboxDataset(BboxDataset):
-    def __init__(self, boxes=None, image_size=None, priors=None, center_variance=0.1, size_variance=0.2,
-                 gt_overlap_tolerance=0.5, expect_data_type=ExpectDataType.absolute_bbox, class_names=None,
-                 symbol='bbox', name=''):
-        super().__init__(boxes=boxes, image_size=image_size, expect_data_type=expect_data_type, class_names=class_names,
-                         symbol=symbol, name=name)
-        self.priors = priors
-        self.center_variance = center_variance
-        self.size_variance = size_variance
-        self.label_transform_funcs = []
-        self.gt_overlap_tolerance = gt_overlap_tolerance
-        self.bbox_post_transform_funcs = []
 
-    def area_of(self, left_top, right_bottom):
-        """Compute the areas of rectangles given two corners.
-
-        Args:
-            left_top (N, 2): left top corner.
-            right_bottom (N, 2): right bottom corner.
-
-        Returns:
-            area (N): return the area.
-        """
-        hw = np.clip(right_bottom - left_top, 0.0, None)
-        return hw[..., 0] * hw[..., 1]
-
-    def iou_of(self, boxes0, boxes1, eps=1e-5):
-        """Return intersection-over-union (Jaccard index) of boxes.
-
-        Args:
-            boxes0 (N, 4): ground truth boxes.
-            boxes1 (N or 1, 4): predicted boxes.
-            eps: a small number to avoid 0 as denominator.
-        Returns:
-            iou (N): IoU values.
-        """
-        overlap_left_top = np.maximum(boxes0[..., :2], boxes1[..., :2])
-        overlap_right_bottom = np.minimum(boxes0[..., 2:], boxes1[..., 2:])
-
-        overlap_area = self.area_of(overlap_left_top, overlap_right_bottom)
-        area0 = self.area_of(boxes0[..., :2], boxes0[..., 2:])
-        area1 = self.area_of(boxes1[..., :2], boxes1[..., 2:])
-        return overlap_area / (area0 + area1 - overlap_area + eps)
-
-    def convert_boxes_to_locations(self, center_form_boxes, center_form_priors):
-        if len(center_form_priors.shape) + 1 == len(center_form_boxes.shape):
-            center_form_priors = np.expand_dims(center_form_priors, 0)
-        return np.concatenate([(center_form_boxes[..., :2] - center_form_priors[..., :2]) / center_form_priors[...,
-                                                                                            2:] / self.center_variance,
-                               np.log(center_form_boxes[..., 2:] / center_form_priors[..., 2:]) / self.size_variance],
-                              axis=len(center_form_boxes.shape) - 1)
-
-    def assign_priors(self, gt_boxes, gt_labels, center_form_priors, iou_threshold):
-        # """Assign ground truth boxes and targets to priors.
-        #
-        # Args:
-        #     gt_boxes (num_targets, 4): ground truth boxes.
-        #     gt_labels (num_targets): labels of targets.
-        #     priors (num_priors, 4): corner form priors
-        # Returns:
-        #     boxes (num_priors, 4): real values for priors.
-        #     labels (num_priros): labels for priors.
-        # """
-        # # size: num_priors x num_targets
-        # # if gt_boxes is not None :
-        #
-        # ious = self.iou_of(gt_boxes.unsqueeze(0), corner_form_priors.unsqueeze(1))
-        # # size: num_priors
-        # best_target_per_prior, best_target_per_prior_index = ious.max(1)
-        # # size: num_targets
-        # best_prior_per_target, best_prior_per_target_index = ious.max(0)
-        #
-        # for target_index, prior_index in enumerate(best_prior_per_target_index):
-        #     best_target_per_prior_index[prior_index] = target_index
-        # # 2.0 is used to make sure every target has a prior assigned
-        # best_target_per_prior.index_fill_(0, best_prior_per_target_index, 2)
-        # # size: num_priors
-        # labels = gt_labels[best_target_per_prior_index]
-        # labels[best_target_per_prior < iou_threshold] = 0  # the backgournd id
-        # boxes = gt_boxes[best_target_per_prior_index]
-        # return boxes, labels
-        corner_form_priors = xywh_to_xyxy(center_form_priors)
-        ious = self.iou_of(np.expand_dims(gt_boxes, 0), np.expand_dims(corner_form_priors, 1))
-        # size: num_priors
-        best_target_per_prior, best_target_per_prior_index = np.max(ious, axis=1), np.argmax(ious, axis=1)
-        # size: num_targets
-        best_prior_per_target, best_prior_per_target_index = np.max(ious, axis=0), np.argmax(ious, axis=0)
-        for target_index, prior_index in enumerate(best_prior_per_target_index):
-            best_target_per_prior_index[prior_index] = target_index
-        # 2.0 is used to make sure every target has a prior assigned
-        best_prior_per_target_index_list = best_prior_per_target_index.tolist()
-        for i in range(best_target_per_prior.shape[0]):
-            if i in best_prior_per_target_index_list:
-                best_target_per_prior[i] = 2
-        labels = gt_labels[best_target_per_prior_index]
-        labels[best_target_per_prior < iou_threshold] = 0  # the backgournd id
-        boxes = gt_boxes[best_target_per_prior_index]
-        return boxes, labels
-
-    def binding_class_names(self, class_names=None, language=None):
-        if class_names is not None and hasattr(class_names, '__len__'):
-            if language is None:
-                language = 'en-us'
-            self.class_names[language] = list(class_names)
-            self.__default_language__ = language
-            self._lab2idx = {v: k for k, v in enumerate(self.class_names[language])}
-            self._idx2lab = {k: v for k, v in enumerate(self.class_names[language])}
-
-    def bbox_transform(self, bbox):
-        if bbox is None or len(bbox) == 0:
-            return np.zeros((self.priors.shape[0], 4)).astype(np.float32), np.zeros((self.priors.shape[0])).astype(
-                np.int64)
-        elif isinstance(bbox, np.ndarray):
+class LandmarkDataset(Dataset):
+    def __init__(self, landmarks=None, image_size=None, expect_data_type=ExpectDataType.landmarks,symbol=None, name=''):
+        super().__init__(symbol=symbol, expect_data_type=expect_data_type, name=name)
+        self.__add__(landmarks)
+        self.dtype = np.float32
+        self.image_size = image_size
+    def __getitem__(self, index: int):
+        self._current_idx = index
+        landmarks = super().__getitem__(index).astype(np.float32)
+        if  (landmarks>1).any() and (self.image_size is None):
+            raise RuntimeError('You need provide image size information for calculate landmarks. ')
+        elif (landmarks>1).any() :
             height, width = self.image_size
-            bbox[:, 0] = bbox[:, 0] / width
-            bbox[:, 2] = bbox[:, 2] / width
-            bbox[:, 1] = bbox[:, 1] / height
-            bbox[:, 3] = bbox[:, 3] / height
-            if bbox.shape[-1] == 5:
-                gt_box = bbox[:, :4]
-                gt_label = bbox[:, 4]
-                boxes, labels = self.assign_priors(gt_box, gt_label, to_numpy(self.priors), 0.3)
-                boxes = xyxy_to_xywh(boxes)
-                locations = self.convert_boxes_to_locations(boxes, self.priors)
-                return boxes.astype(np.float32), labels.astype(np.int64)
-            else:
-                return bbox
+            landmarks[:, 0] = landmarks[:, 0] / width
+            landmarks[:, 1] = landmarks[:, 1] / height
+            return np.array(landmarks).astype(np.float32)
+        else:
+            return np.array(landmarks).astype(np.float32)
+
+
+
+
 
 
 class NumpyDataset(Dataset):
@@ -511,13 +564,11 @@ class Iterator(object):
         self._unpair = NumpyDataset()
         self.workers = 2
         self.itr = 0
-        if data is not None and isinstance(data, Dataset):
+        if data is not None and isinstance(data, (Dataset,MultipleDataset)):
             self._data = data
-        if label is not None and isinstance(label, Dataset):
+        if label is not None and isinstance(label, (Dataset,MultipleDataset)):
             self._label = label
-            if isinstance(self._label, (MaskDataset, ImageDataset, BboxDataset)) and isinstance(self._data,
-                                                                                                ImageDataset) and len(
-                self._label) == len(self._data):
+            if isinstance(self._label, (MaskDataset, ImageDataset, BboxDataset,MultipleDataset)) and isinstance(self._data, ImageDataset) and len( self._label) == len(self._data):
                 self._label.is_pair_process = self._data.is_pair_process = self.is_pair_process = True
             else:
                 self._label.is_pair_process = self._data.is_pair_process = self.is_pair_process = False
@@ -590,30 +641,24 @@ class Iterator(object):
 
     def update_signature(self, arg_names):
         iterdata = self.next()
-        if self.signature is None:
-            self.signature = OrderedDict()
+        if self.signature is None or not isinstance(self.signature,Signature):
+            self.signature = Signature()
+            self.signature.name = 'data_provider'
+        if isinstance(arg_names, (list, tuple)) and len(iterdata)==len(arg_names):
             for i in range(len(arg_names)):
                 arg = arg_names[i]
-                data = iterdata[i][0]
-                self.signature[arg] = data.shape
-        elif len(self.signature.key_list) == len(arg_names):
-            self.signature = OrderedDict()
-            for i in range(len(arg_names)):
-                arg = arg_names[i]
-                data = iterdata[i][0]
-                self.signature[arg] = data.shape
+                data = iterdata[i]
+                self.signature.outputs[arg] =(-1,)+ data.shape[1:] if data.ndim>1 else (-1)
 
         elif not isinstance(arg_names, (list, tuple)):
             raise ValueError('arg_names should be list or tuple')
         elif len(self.signature.key_list) != len(arg_names):
-            raise ValueError('data deed and arg_names should be the same length')
+            raise ValueError('data feed and arg_names should be the same length')
         else:
-            self.signature = OrderedDict()
+            self.signature = None
             iterdata = self.next()
-            for i in range(len(arg_names)):
-                arg = arg_names[i]
-                data = iterdata[i][0]
-                self.signature[arg] = data.shape
+
+
 
     def paired_transform(self, img_data, paired_img):
 
@@ -642,15 +687,16 @@ class Iterator(object):
         try:
             bbox = None
             mask = None
-            data = self.data.__getitem__(index % len(self.data)) if len(self.data) > 0 else None
+            data = self.data.__getitem__(index % len(self.data)) if self.data is not None and len(self.data) > 0 else None
             # stop = time.time()
             # print('get data:{0}'.format(stop - start))
             # start=stop
-            label = self.label.__getitem__(index % len(self.label)) if len(self.label) > 0 else None
+
+            label = self.label.__getitem__(index % len(self.label)) if self.label is not None and len(self.label) > 0 else None
             # stop = time.time()
             # print('get label:{0}'.format(stop - start))
             # start = stop
-            if isinstance(self.label, BboxDataset):
+            if isinstance(self.label, (BboxDataset,LandmarkDataset)):
                 data, label = self.paired_transform(data, label)
                 if hasattr(self.data, 'image_transform'):
                     data = self.data.image_transform(data)
@@ -681,7 +727,7 @@ class Iterator(object):
                 # print('paired_transform:{0}'.format(stop - start))
                 # start = stop
                 if hasattr(self.data, 'image_transform'):
-                    data = self.data.image_transform( data)  # stop = time.time()  # print('data image_transform:{0}'.format(stop - start))  #
+                    data = self.data.image_transform(data) # stop = time.time()  # print('data image_transform:{0}'.format(stop - start))  #
                     # start = stop
                 if hasattr(self.label, 'image_transform'):
                     label = self.label.image_transform(label)  # stop = time.time()  # print('label image_transform:{0}'.format(stop - start))  #
@@ -697,31 +743,28 @@ class Iterator(object):
 
             return_data = []
             if self.signature is None or len(self.signature) == 0:
-                self.signature = OrderedDict()
+                self.signature=Signature()
+                self.signature.name='data_provider'
                 if data is not None:
-                    self.signature['data' if self.data.symbol is None or len(
-                        self.data.symbol) == 0 else self.data.symbol] = data.shape
+                    self.signature.outputs['data' if self.data.symbol is None or len(self.data.symbol) == 0 else self.data.symbol] = (-1,)+data.shape
                 if bbox is not None:
-                    self.signature['bbox' if self.label.symbol is None or len(
-                        self.label.symbol) == 0 else self.label.symbol] = bbox.shape
+                    self.signature.outputs['bbox' if self.label.symbol is None or len(self.label.symbol) == 0 else self.label.symbol] =(-1,)+ bbox.shape
                 if mask is not None:
-                    self.signature['mask' if self.label.symbol is None or len(
-                        self.label.symbol) == 0 else self.label.symbol] = mask.shape
+                    self.signature.outputs['mask' if self.label.symbol is None or len(self.label.symbol) == 0 else self.label.symbol] = (-1,)+mask.shape
                 if label is not None:
-                    self.signature['label' if self.label.symbol is None or len(self.label.symbol) == 0 else self.label.symbol] = label.shape if isinstance(label, np.ndarray) else type(label)
+                    self.signature.outputs['label' if self.label.symbol is None or len(self.label.symbol) == 0 or self.label.symbol in self.signature else self.label.symbol] =(-1,)+ label.shape if isinstance(label, np.ndarray) else (-1,)
                 if unpair is not None:
-                    self.signature['unpair' if self.unpair.symbol is None or len(
-                        self.unpair.symbol) == 0 else self.unpair.symbol] = unpair.shape  # stop = time.time()  #
+                    self.signature.outputs['unpair' if self.unpair.symbol is None or len(self.unpair.symbol) == 0 else self.unpair.symbol] = (-1,)+unpair.shape  # stop = time.time()  #
                     # print('signature:{0}'.format(stop - start))  # start = stop
 
             if data is not None:
                 return_data.append(data)
-            if label is not None:
-                return_data.append(label)
             if bbox is not None:
                 return_data.append(bbox)
             if mask is not None:
                 return_data.append(mask)
+            if label is not None:
+                return_data.append(label)
             if unpair is not None:
                 return_data.append(unpair)
             # stop = time.time()
