@@ -1,3 +1,7 @@
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import print_function
+
 try:
     import Queue
 except ImportError:
@@ -5,7 +9,7 @@ except ImportError:
 import threading
 import time
 
-from ..misc.ipython_utils import is_in_ipython, is_in_colab
+from trident.misc.ipython_utils import is_in_ipython, is_in_colab
 
 if is_in_ipython():
     from IPython import display
@@ -17,7 +21,7 @@ else:
     import matplotlib
 import matplotlib.pyplot as plt
 import itertools
-from .image_common import list_pictures
+from trident.data.image_common import list_pictures
 
 
 
