@@ -1,3 +1,4 @@
+"""Pytorch pooling layers definition in trident"""
 from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
@@ -20,10 +21,10 @@ from torch.nn import Module
 from torch.nn import init
 from torch.nn.parameter import Parameter
 
-from ..backend.common import *
-from ..backend.pytorch_backend import to_numpy, to_tensor, Layer, Sequential
+from trident.backend.common import *
+from trident.backend.pytorch_backend import Layer, Sequential
 
-__all__ = ['MaxPool2d', 'MaxPool1d', 'MaxPool3d', 'MaxUnpool1d', 'MaxUnpool2d', 'MaxUnpool3d', 'AvgPool1d', 'AvgPool2d',
+__all__ = [ 'MaxPool1d','MaxPool2d', 'MaxPool3d', 'MaxUnpool1d', 'MaxUnpool2d', 'MaxUnpool3d', 'AvgPool1d', 'AvgPool2d',
            'AvgPool3d', 'GlobalAvgPool2d', 'AdaptiveAvgPool2d']
 
 _session = get_session()
@@ -74,8 +75,7 @@ class _PoolNd(Layer):
 
 
 class MaxPool1d(_PoolNd):
-    r"""Applies a 1D max pooling over an input signal composed of several input
-    planes.
+    """Applies a 1D max pooling over an input signal composed of several input planes.
 
     In the simplest case, the output value of the layer with input size :math:`(N, C, L)`
     and output :math:`(N, C, L_{out})` can be precisely described as:
@@ -100,20 +100,18 @@ class MaxPool1d(_PoolNd):
     Shape:
         - Input: :math:`(N, C, L_{in})`
         - Output: :math:`(N, C, L_{out})`, where
+        : :math:'L_{out} = \left\lfloor \frac{L_{in} + 2 \times \text{padding} - \text{dilation}\times (\text{kernel\_size} - 1) - 1}{\text{stride}} + 1\right\rfloor'
 
-          .. math::
-              L_{out} = \left\lfloor \frac{L_{in} + 2 \times \text{padding} - \text{dilation}
-                    \times (\text{kernel\_size} - 1) - 1}{\text{stride}} + 1\right\rfloor
-
-    Examples::
+    Examples:
 
         >>> # pool of size=3, strides=2
         >>> m = nn.MaxPool1d(3, strides=2)
         >>> input = torch.randn(20, 16, 50)
         >>> output = m(input)
 
-    .. _link:
+    References:
         https://github.com/vdumoulin/conv_arithmetic/blob/master/README.md
+
     """
 
     def __init__(self, kernel_size, strides=None, auto_pad=True, padding_mode='replicate', name='', **kwargs):
@@ -158,9 +156,9 @@ class MaxPool2d(_PoolNd):
 
     Args:
         kernel_size: the size of the window to take a max over
-        stride: the strides of the window. Default value is :attr:`kernel_size`
+        stride: the stride of the window. Default value is :attr:`kernel_size`
         padding: implicit zero padding to be added on both sides
-        dilation: a parameter that controls the strides of elements in the window
+        dilation: a parameter that controls the stride of elements in the window
         return_indices: if ``True``, will return the max indices along with the outputs.
                         Useful for :class:`torch.nn.MaxUnpool2d` later
         ceil_mode: when True, will use `ceil` instead of `floor` to compute the output shape
@@ -177,12 +175,12 @@ class MaxPool2d(_PoolNd):
               W_{out} = \left\lfloor\frac{W_{in} + 2 * \text{padding[1]} - \text{dilation[1]}
                     \times (\text{kernel\_size[1]} - 1) - 1}{\text{stride[1]}} + 1\right\rfloor
 
-    Examples::
+    Examples:
 
-        >>> # pool of square window of size=3, strides=2
-        >>> m = nn.MaxPool2d(3, strides=2)
+        >>> # pool of square window of size=3, stride=2
+        >>> m = nn.MaxPool2d(3, stride=2)
         >>> # pool of non-square window
-        >>> m = nn.MaxPool2d((3, 2), strides=(2, 1))
+        >>> m = nn.MaxPool2d((3, 2), stride=(2, 1))
         >>> input = torch.randn(20, 16, 50, 32)
         >>> output = m(input)
 
@@ -190,6 +188,7 @@ class MaxPool2d(_PoolNd):
         https://github.com/vdumoulin/conv_arithmetic/blob/master/README.md
     """
 
+
     def __init__(self, kernel_size, strides=None, auto_pad=True, padding_mode='zero', name='', **kwargs):
         super(MaxPool2d, self).__init__(kernel_size, strides, auto_pad, padding_mode, 1, name, **kwargs)
         self.kernel_size = _pair(kernel_size)
@@ -210,7 +209,7 @@ class MaxPool2d(_PoolNd):
             kh, kw = self.kernel_size[-2:]
             sh, sw = self.strides[-2:]
 
-            oh, ow = math.ceil(ih / sh), math.ceil(iw / sw)
+            oh, ow = math.ceil(ih // sh), math.ceil(iw // sw)
             pad_h = max((oh - 1) * sh + (kh - 1) + 1 - ih, 0)
             pad_w = max((ow - 1) * sw + (kw - 1) + 1 - iw, 0)
             if pad_h % 2 == 1 and sh > 1:
@@ -222,7 +221,7 @@ class MaxPool2d(_PoolNd):
             pad_h = self.padding[0] * 2
             pad_w = self.padding[1] * 2
 
-        self.padding = (int(pad_h / 2), int(pad_w / 2))
+        self.padding = (int(pad_h // 2), int(pad_w // 2))
 
     def forward(self, *x):
         x = enforce_singleton(x)
@@ -235,7 +234,7 @@ class MaxPool2d(_PoolNd):
 
 
 class MaxPool3d(_PoolNd):
-    r"""Applies a 3D max pooling over an input signal composed of several input
+    """Applies a 3D max pooling over an input signal composed of several input
     planes.
 
     In the simplest case, the output value of the layer with input size :math:`(N, C, D, H, W)`,
@@ -244,10 +243,10 @@ class MaxPool3d(_PoolNd):
 
     .. math::
         \begin{aligned}
-            \text{out}(N_i, C_j, d, h, w) ={} & \max_{k=0, \ldots, kD-1} \max_{m=0, \ldots, kH-1} \max_{n=0, \ldots, 
+            \text{out}(N_i, C_j, d, h, w) ={} & \max_{k=0, \ldots, kD-1} \max_{m=0, \ldots, kH-1} \max_{n=0, \ldots,
             kW-1} \\
                                               & \text{input}(N_i, C_j, \text{stride[0]} \times d + k,
-                                                             \text{stride[1]} \times h + m, \text{stride[2]} \times w 
+                                                             \text{stride[1]} \times h + m, \text{stride[2]} \times w
                                                              + n)
         \end{aligned}
 
@@ -286,7 +285,7 @@ class MaxPool3d(_PoolNd):
               W_{out} = \left\lfloor\frac{W_{in} + 2 \times \text{padding}[2] - \text{dilation}[2] \times
                 (\text{kernel\_size}[2] - 1) - 1}{\text{stride}[2]} + 1\right\rfloor
 
-    Examples::
+    Examples:
 
         >>> # pool of square window of size=3, strides=2
         >>> m = nn.MaxPool3d(3, strides=2)
@@ -297,7 +296,8 @@ class MaxPool3d(_PoolNd):
 
     .. _link:
         https://github.com/vdumoulin/conv_arithmetic/blob/master/README.md
-    """  # noqa: E501
+
+    """
 
     def __init__(self, kernel_size, strides=None, auto_pad=True, name='', **kwargs):
         super(MaxPool3d, self).__init__(kernel_size, strides, auto_pad, 1, name, **kwargs)
@@ -313,7 +313,7 @@ class MaxPool3d(_PoolNd):
 
 
 class MaxUnpool1d(_PoolNd):
-    r"""Computes a partial inverse of :class:`MaxPool1d`.
+    """Computes a partial inverse of :class:`MaxPool1d`.
 
     :class:`MaxPool1d` is not fully invertible, since the non-maximal values are lost.
 
@@ -347,7 +347,7 @@ class MaxUnpool1d(_PoolNd):
 
           or as given by :attr:`output_size` in the call operator
 
-    Example::
+    Examples:
 
         >>> pool = nn.MaxPool1d(2, strides=2, return_indices=True)
         >>> unpool = nn.MaxUnpool1d(2, strides=2)
@@ -364,6 +364,7 @@ class MaxUnpool1d(_PoolNd):
 
         >>> unpool(output, indices)
         tensor([[[ 0.,  2.,  0.,  4.,  0.,  6.,  0., 8.]]])
+
     """
 
     def __init__(self, kernel_size, strides=None, auto_pad=True, name='', **kwargs):
@@ -377,7 +378,7 @@ class MaxUnpool1d(_PoolNd):
 
 
 class MaxUnpool2d(_PoolNd):
-    r"""Computes a partial inverse of :class:`MaxPool2d`.
+    """Computes a partial inverse of :class:`MaxPool2d`.
 
     :class:`MaxPool2d` is not fully invertible, since the non-maximal values are lost.
 
@@ -414,7 +415,7 @@ class MaxUnpool2d(_PoolNd):
 
           or as given by :attr:`output_size` in the call operator
 
-    Example::
+    Examples:
 
         >>> pool = nn.MaxPool2d(2, strides=2, return_indices=True)
         >>> unpool = nn.MaxUnpool2d(2, strides=2)
@@ -436,6 +437,7 @@ class MaxUnpool2d(_PoolNd):
                   [  0.,   0.,   0.,  14.,   0.],
                   [ 16.,   0.,   0.,   0.,   0.],
                   [  0.,   0.,   0.,   0.,   0.]]]])
+
     """
 
     def __init__(self, kernel_size, strides=None, auto_pad=True, name='', **kwargs):
@@ -451,7 +453,7 @@ class MaxUnpool2d(_PoolNd):
 
 
 class MaxUnpool3d(_PoolNd):
-    r"""Computes a partial inverse of :class:`MaxPool3d`.
+    """Computes a partial inverse of :class:`MaxPool3d`.
 
     :class:`MaxPool3d` is not fully invertible, since the non-maximal values are lost.
     :class:`MaxUnpool3d` takes in as input the output of :class:`MaxPool3d`
@@ -490,7 +492,7 @@ class MaxUnpool3d(_PoolNd):
 
           or as given by :attr:`output_size` in the call operator
 
-    Example::
+    Examples:
 
         >>> # pool of square window of size=3, strides=2
         >>> pool = nn.MaxPool3d(3, strides=2, return_indices=True)
@@ -499,6 +501,7 @@ class MaxUnpool3d(_PoolNd):
         >>> unpooled_output = unpool(output, indices)
         >>> unpooled_output.size()
         torch.Size([20, 16, 51, 33, 15])
+
     """
 
     def __init__(self, kernel_size, strides=None, auto_pad=True, name='', **kwargs):
@@ -512,7 +515,7 @@ class MaxUnpool3d(_PoolNd):
 
 
 class AvgPool1d(_PoolNd):
-    r"""Applies a 1D average pooling over an input signal composed of several
+    """Applies a 1D average pooling over an input signal composed of several
     input planes.
 
     In the simplest case, the output value of the layer with input size :math:`(N, C, L)`,
@@ -545,12 +548,13 @@ class AvgPool1d(_PoolNd):
               L_{out} = \left\lfloor \frac{L_{in} +
               2 \times \text{padding} - \text{kernel\_size}}{\text{stride}} + 1\right\rfloor
 
-    Examples::
+    Examples:
 
         >>> # pool with window of size=3, strides=2
         >>> m = nn.AvgPool1d(3, strides=2)
         >>> m(torch.tensor([[[1.,2,3,4,5,6,7]]]))
         tensor([[[ 2.,  4.,  6.]]])
+
     """
 
     def __init__(self, kernel_size, strides=None, auto_pad=True, name='', **kwargs):
@@ -566,7 +570,7 @@ class AvgPool1d(_PoolNd):
 
 
 class AvgPool2d(_PoolNd):
-    r"""Applies a 2D average pooling over an input signal composed of several input
+    """Applies a 2D average pooling over an input signal composed of several input
     planes.
 
     In the simplest case, the output value of the layer with input size :math:`(N, C, H, W)`,
@@ -607,7 +611,7 @@ class AvgPool2d(_PoolNd):
               W_{out} = \left\lfloor\frac{W_{in}  + 2 \times \text{padding}[1] -
                 \text{kernel\_size}[1]}{\text{stride}[1]} + 1\right\rfloor
 
-    Examples::
+    Examples:
 
         >>> # pool of square window of size=3, strides=2
         >>> m = nn.AvgPool2d(3, strides=2)
@@ -658,7 +662,7 @@ class AvgPool2d(_PoolNd):
 
 
 class AvgPool3d(_PoolNd):
-    r"""Applies a 3D average pooling over an input signal composed of several input
+    """Applies a 3D average pooling over an input signal composed of several input
     planes.
 
     In the simplest case, the output value of the layer with input size :math:`(N, C, D, H, W)`,
@@ -706,7 +710,7 @@ class AvgPool3d(_PoolNd):
               W_{out} = \left\lfloor\frac{W_{in} + 2 \times \text{padding}[2] -
                     \text{kernel\_size}[2]}{\text{stride}[2]} + 1\right\rfloor
 
-    Examples::
+    Examples:
 
         >>> # pool of square window of size=3, strides=2
         >>> m = nn.AvgPool3d(3, strides=2)
@@ -738,7 +742,14 @@ class AvgPool3d(_PoolNd):
 
 
 class GlobalAvgPool2d(Layer):
+    """Global Average Pooling Imprementation """
     def __init__(self, keepdim=False, name='avg_pool'):
+        """
+
+        Args:
+            keepdim ():
+            name ():
+        """
         super(GlobalAvgPool2d, self).__init__()
         self.keepdim = keepdim
         self.name = name
@@ -756,15 +767,13 @@ class GlobalAvgPool2d(Layer):
 
     def forward(self, *x):
         x = enforce_singleton(x)
-        if x.size(1) == 1:
-            x = x.view(x.size(0), x.size(1), -1).mean(dim=-1, keepdim=self.keepdim)
-
-        else:
-            x = x.view(x.size(0), x.size(1), -1).mean(dim=-1, keepdim=self.keepdim)
+        N,C,H,W=x.size()
+        x = x.view(N, C, -1).mean(dim=-1, keepdim=self.keepdim)
         return x
 
 
 class AdaptiveAvgPool2d(Layer):
+    """AdaptiveAverage Pooling Imprementation """
     def __init__(self, output_size, name='adaptive_avg_pool'):
         super(AdaptiveAvgPool2d, self).__init__()
         self.output_size = _pair(output_size)
@@ -774,30 +783,3 @@ class AdaptiveAvgPool2d(Layer):
         x = enforce_singleton(x)
         return F.adaptive_avg_pool2d(x, self.output_size)
 
-#
-# class GlobalAvgPool1d(Layer):
-#     def __init__(self):
-#         super(GlobalAvgPool1d, self).__init__()
-#
-#     def build(self, input_shape):
-#         pass
-#
-#     def forward(self, *x):
-#         x = enforce_singleton(x)
-#         assert len(x.size()) == 3, x.size()
-#         B, C, L = x.size()
-#         return F.avg_pool1d(x, L)
-#
-#
-# class GlobalAvgPool2d(Layer):
-#     def __init__(self):
-#         super(GlobalAvgPool2d, self).__init__()
-#
-#     def build(self, input_shape):
-#         pass
-#
-#     def forward(self, *x):
-#         x = enforce_singleton(x)
-#         assert len(x.size()) == 4, x.size()
-#         B, C, W, H = x.size()
-#         return F.avg_pool2d(x, (W, H))
