@@ -20,16 +20,16 @@ from torch._six import container_abcs
 from torch.nn import init
 from torch.nn.parameter import Parameter
 
-from ..backend.common import *
-from ..backend.pytorch_backend import to_numpy, to_tensor, Layer, Sequential
-from ..data.image_common import *
-from ..data.utils import download_model_from_google_drive
-from ..layers.pytorch_activations import get_activation, Identity
-from ..layers.pytorch_blocks import *
-from ..layers.pytorch_layers import *
-from ..layers.pytorch_normalizations import get_normalization
-from ..layers.pytorch_pooling import *
-from ..optims.pytorch_trainer import *
+from trident.backend.common import *
+from trident.backend.pytorch_backend import to_numpy, to_tensor, Layer, Sequential
+from trident.data.image_common import *
+from trident.data.utils import download_model_from_google_drive
+from trident.layers.pytorch_activations import get_activation, Identity
+from trident.layers.pytorch_blocks import *
+from trident.layers.pytorch_layers import *
+from trident.layers.pytorch_normalizations import get_normalization
+from trident.layers.pytorch_pooling import *
+from trident.optims.pytorch_trainer import *
 
 __all__ = ['MobileNet','MobileNetV2']
 
@@ -55,9 +55,10 @@ def _make_divisible(v, divisor, min_value=None):
     It ensures that all layers have a channel number that is divisible by 8
     It can be seen here:
     https://github.com/tensorflow/models/blob/master/research/slim/nets/mobilenet/mobilenet.py
-    :param v:
-    :param divisor:
-    :param min_value:
+    Args
+        v:
+        divisor:
+        min_value:
     :return:
     """
     if min_value is None:
@@ -109,7 +110,7 @@ def MobileNet( input_shape=(3, 224, 224), classes=1000, use_bias=False, width_mu
         mobilenet.add_module('fc',Dense((classes),activation=None))
         mobilenet.add_module('softmax', SoftMax(name='softmax'))
     model = ImageClassificationModel(input_shape=input_shape, output=mobilenet)
-    model.signature=get_signature(model.model.forward)
+
     with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'imagenet_labels1.txt'), 'r',
               encoding='utf-8-sig') as f:
         labels = [l.rstrip() for l in f]
@@ -139,10 +140,15 @@ def MobileNetV2(include_top=True,
         recovery_model.to(_device)
         if include_top==False:
             recovery_model.__delitem__(-1)
+            recovery_model.__delitem__(-1)
+            recovery_model.__delitem__(-1)
+            recovery_model.__delitem__(-1)
+            mob.class_names = []
         else:
             if classes!=1000:
                 new_fc = Dense(classes, activation=None, name='fc')
                 new_fc.input_shape=recovery_model.fc.input_shape
                 recovery_model.fc=new_fc
+                mob.class_names=[]
         mob.model=recovery_model
     return mob
\ No newline at end of file
