@@ -17,26 +17,26 @@ import numpy as np
 import torch
 import torch.nn as nn
 import torch.nn.functional as F
-import torchvision
+
 from matplotlib.collections import PolyCollection
 from torch._six import container_abcs
 from torch.nn import init
 from torch.nn.parameter import Parameter
 
-from ..backend.common import *
-from ..backend.pytorch_backend import *
-from ..backend.pytorch_backend import to_numpy, to_tensor, Layer, Sequential, Combine
-from ..backend.pytorch_ops import meshgrid
-from ..data.bbox_common import clip_boxes_to_image, nms
-from ..data.image_common import *
-from ..data.utils import download_model_from_google_drive
-from ..layers.pytorch_activations import get_activation, Identity, PRelu
-from ..layers.pytorch_blocks import *
-from ..layers.pytorch_layers import *
-from ..layers.pytorch_normalizations import get_normalization
-from ..layers.pytorch_pooling import *
-from ..optims.pytorch_trainer import *
-from ..optims.pytorch_trainer import ImageDetectionModel
+from trident.backend.common import *
+from trident.backend.pytorch_backend import *
+from trident.backend.pytorch_backend import to_numpy, to_tensor, Layer, Sequential, Combine
+from trident.backend.pytorch_ops import *
+from trident.data.bbox_common import clip_boxes_to_image, nms
+from trident.data.image_common import *
+from trident.data.utils import download_model_from_google_drive
+from trident.layers.pytorch_activations import get_activation, Identity, PRelu
+from trident.layers.pytorch_blocks import *
+from trident.layers.pytorch_layers import *
+from trident.layers.pytorch_normalizations import get_normalization
+from trident.layers.pytorch_pooling import *
+from trident.optims.pytorch_trainer import *
+from trident.optims.pytorch_trainer import ImageDetectionModel
 
 __all__ = ['Pnet','Rnet','Onet','Mtcnn']
 
@@ -56,60 +56,64 @@ if not os.path.exists(dirname):
         pass
 
 
-p_net=Sequential(
+def p_net():
+    return Sequential(
     Conv2d((3,3),10,strides=1,auto_pad=False,use_bias=True,name='conv1'),
-    PRelu(),
+    PRelu(num_parameters=1),
     MaxPool2d((2,2),strides=2,auto_pad=False),
     Conv2d((3, 3), 16, strides=1, auto_pad=False,use_bias=True,name='conv2'),
-    PRelu(),
+    PRelu(num_parameters=1),
     Conv2d((3,3),32,strides=1,auto_pad=False,use_bias=True,name='conv3'),
-    PRelu(),
+    PRelu(num_parameters=1),
     Combine(
         Conv2d((1,1),1,strides=1,auto_pad=False,use_bias=True,activation='sigmoid',name='conv4_1'),
         Conv2d((1,1),4,strides=1,auto_pad=False,use_bias=True,name='conv4_2'),
-        Conv2d((1,1),10,strides=1,auto_pad=False,use_bias=True,name='conv4_3')),name='detector_head')
-p_net.name='pnet'
+        Conv2d((1,1),10,strides=1,auto_pad=False,use_bias=True,name='conv4_3')),name='pnet')
 
 
-r_net=Sequential(
+
+def r_net():
+    return Sequential(
     Conv2d((3,3),28,strides=1,auto_pad=False,use_bias=True,name='conv1'),
-    PRelu(),
+    PRelu(num_parameters=1),
     MaxPool2d((3,3),strides=2,auto_pad=False),
     Conv2d((3, 3), 48, strides=1, auto_pad=False,use_bias=True,name='conv2'),
-    PRelu(),
+    PRelu(num_parameters=1),
     MaxPool2d((3,3),strides=2,auto_pad=False),
     Conv2d((2,2),64,strides=1,auto_pad=False,use_bias=True,name='conv3'),
-    PRelu(),
+    PRelu(num_parameters=1),
     Flatten(),
     Dense(128,activation=None,use_bias=True,name='conv4'),
-    PRelu(),
+    PRelu(num_parameters=1),
     Combine(
         Dense(1,activation='sigmoid',use_bias=True,name='conv5_1'),
         Dense(4,activation=None,use_bias=True,name='conv5_2'),
-        Dense(10,activation=None,use_bias=True,name='conv5_3')),name='detector_head')
-r_net.name='rnet'
+        Dense(10,activation=None,use_bias=True,name='conv5_3'))
+    ,name='rnet')
 
 
-o_net=Sequential(
+
+def o_net():
+    return Sequential(
     Conv2d((3,3),32,strides=1,auto_pad=False,use_bias=True,name='conv1'),
-    PRelu(),
+    PRelu(num_parameters=1),
     MaxPool2d((3,3),strides=2,auto_pad=False),
     Conv2d((3, 3), 64, strides=1, auto_pad=False,use_bias=True,name='conv2'),
-    PRelu(),
+    PRelu(num_parameters=1),
     MaxPool2d((3,3),strides=2,auto_pad=False),
     Conv2d((3,3),64,strides=1,auto_pad=False,use_bias=True,name='conv3'),
-    PRelu(),
+    PRelu(num_parameters=1),
     MaxPool2d((2, 2), strides=2,auto_pad=False),
     Conv2d((2, 2), 128, strides=1, auto_pad=False,use_bias=True,name='conv4'),
-    PRelu(),
+    PRelu(num_parameters=1),
     Flatten(),
     Dense(256,activation=None,use_bias=True,name='conv5'),
-    PRelu(),
+    PRelu(num_parameters=1),
     Combine(
         Dense(1,activation='sigmoid',use_bias=True,name='conv6_1'),
         Dense(4,activation=None,use_bias=True,name='conv6_2'),
-        Dense(10,activation=None,use_bias=True,name='conv6_3')),name='detector_head')
-o_net.name='onet'
+        Dense(10,activation=None,use_bias=True,name='conv6_3')),name='onet')
+
 
 
 def Pnet(pretrained=True,
@@ -119,8 +123,8 @@ def Pnet(pretrained=True,
         input_shape=tuple(input_shape)
     else:
         input_shape=(3,12,12)
-    pnet =ImageDetectionModel(input_shape=(3,12,12),output=p_net)
-    pnet.preprocess_flow = [normalize(0, 255), image_backend_adaptive]
+    pnet =ImageDetectionModel(input_shape=(3,12,12),output=p_net())
+    pnet.preprocess_flow = [normalize(0, 255), image_backend_adaption]
     if pretrained==True:
         download_model_from_google_drive('1w9ahipO8D9U1dAXMc2BewuL0UqIBYWSX',dirname,'pnet.pth')
         recovery_model=torch.load(os.path.join(dirname,'pnet.pth'))
@@ -136,8 +140,8 @@ def Rnet(pretrained=True,
         input_shape=tuple(input_shape)
     else:
         input_shape=(3,24,24)
-    rnet =ImageDetectionModel(input_shape=(3,24,24),output=r_net)
-    rnet.preprocess_flow = [normalize(0, 255), image_backend_adaptive]
+    rnet =ImageDetectionModel(input_shape=(3,24,24),output=r_net())
+    rnet.preprocess_flow = [normalize(0, 255), image_backend_adaption]
     if pretrained==True:
         download_model_from_google_drive('1CH7z133_KrcWMx9zXAblMCV8luiQ3wph',dirname,'rnet.pth')
         recovery_model=torch.load(os.path.join(dirname,'rnet.pth'))
@@ -152,8 +156,8 @@ def Onet(pretrained=True,
         input_shape=tuple(input_shape)
     else:
         input_shape=(3,48,48)
-    onet =ImageDetectionModel(input_shape=(3,48,48),output=o_net)
-    onet.preprocess_flow = [normalize(0, 255), image_backend_adaptive]
+    onet =ImageDetectionModel(input_shape=(3,48,48),output=o_net())
+    onet.preprocess_flow = [normalize(0, 255), image_backend_adaption]
     if pretrained==True:
         download_model_from_google_drive('1a1dAlSzJOAfIz77Ic38JMQJYWDG_b7-_',dirname,'onet.pth')
         recovery_model=torch.load(os.path.join(dirname,'onet.pth'))
@@ -169,6 +173,7 @@ class DetectorHead(Layer):
         self.cellsize=cellsize
         self.threshould=threshould
         self.min_size=min_size
+
         self._built =True
 
     def forward(self, input,**kwargs):
@@ -253,24 +258,28 @@ def calibrate_box(bboxes, offsets):
 
 
 class Mtcnn(ImageDetectionModel):
-    def __init__(self, pretrained=True, min_size=10, **kwargs):
-        pnet = ImageDetectionModel(input_shape=(3, 12, 12), output=p_net).model
-        self.rnet = ImageDetectionModel(input_shape=(3, 24, 24), output=r_net).model
-        self.onet = ImageDetectionModel(input_shape=(3, 48, 48), output=o_net).model
+    def __init__(self, pretrained=True, min_size=10,verbose=True, **kwargs):
+        self.verbose = verbose
+        pnet =p_net()
+        self.rnet = ImageDetectionModel(input_shape=(3, 24, 24), output=r_net())._model
+        self.onet = ImageDetectionModel(input_shape=(3, 48, 48), output=o_net())._model
         if pretrained == True:
-            pnet = Pnet().model
-            self.rnet = Rnet().model
-            self.onet = Onet().model
+            pnet =Pnet()._model
+            self.rnet = Rnet()._model
+            self.onet = Onet()._model
         self.min_size = min_size
-        self.signature=get_signature(self.model.forward)
+
 
 
         super(Mtcnn, self).__init__(input_shape=(3,224,224),output=pnet)
         self.pnet=pnet
-        self.model=pnet
+
+        self.signature = get_signature(self._model.forward)
+        #data preprocess
         self.preprocess_flow =[normalize(0,255)]
         self.nms_threshould = [0.9, 0.9, 0.3]
         self.detection_threshould = [0.5, 0.6, 0.9]
+
         pnet.add_module('pnet_detector', DetectorHead(cellsize=12, threshould=0.5, min_size=self.min_size))
 
 
@@ -294,25 +303,101 @@ class Mtcnn(ImageDetectionModel):
                 for func in self.preprocess_flow:
                     if inspect.isfunction(func):
                         scaled_img=func(scaled_img)
-            images.append(image_backend_adaptive(scaled_img))
+            images.append(image_backend_adaption(scaled_img))
             minl = minl * factor
             factor_count += 1
         return images, scales
 
     #adjust bbox like square
-    def rerec(self,bboxA,img_shape):
+    def rerec(self, bboxA, img_shape):
         """Convert bboxA to square."""
-        bboxA=to_numpy(bboxA)
+
         h = bboxA[:, 3] - bboxA[:, 1]
         w = bboxA[:, 2] - bboxA[:, 0]
-        max_len = np.maximum(w, h)
+        max_len = maximum(w, h)
 
+        bboxA[:, 0] = round(bboxA[:, 0] - 0.5 * (max_len - w), 0)
+        bboxA[:, 1] = round(bboxA[:, 1] - 0.5 * (max_len - h), 0)
+        bboxA[:, 2] = bboxA[:, 0] + max_len
+        bboxA[:, 3] = bboxA[:, 1] + max_len
+        return bboxA
 
-        bboxA[:, 0] = bboxA[:, 0] -0.5*(max_len-w)
-        bboxA[:, 1] = bboxA[:, 1] -0.5*(max_len-h)
-        bboxA[:, 2] = bboxA[:, 0]+max_len
-        bboxA[:, 3] =bboxA[:,  1]+max_len
-        return to_tensor(bboxA)
+    # 計算面積
+    def area_of(self,left_top, right_bottom):
+        """Compute the areas of rectangles given two corners.
+
+        Args:
+            left_top (N, 2): left top corner.
+            right_bottom (N, 2): right bottom corner.
+
+        Returns:
+            area (N): return the area.
+        """
+        hw = right_bottom - left_top
+        return clip(hw[..., 0], min=0) * clip(hw[..., 1], min=0)
+
+    # 計算IOU(交集/聯集)
+    def iou_of(self,boxes0, boxes1, eps=1e-5):
+        """Return intersection-over-union (Jaccard index) of boxes.
+
+        Args:
+            boxes0 (N, 4): ground truth boxes.
+            boxes1 (N or 1, 4): predicted boxes.
+            eps: a small number to avoid 0 as denominator.
+        Returns:
+            iou (N): IoU values.
+        """
+        overlap_left_top = maximum(boxes0[..., :2], boxes1[..., :2])
+        overlap_right_bottom = minimum(boxes0[..., 2:], boxes1[..., 2:])
+
+        overlap_area = self.area_of(overlap_left_top, overlap_right_bottom)
+        area0 = self.area_of(boxes0[..., :2], boxes0[..., 2:])
+        area1 = self.area_of(boxes1[..., :2], boxes1[..., 2:])
+        return overlap_area / (area0 + area1 - overlap_area + eps)
+
+    # 基於tensor計算nms
+    def boxes_nms(self,box_scores, overlap_threshold=0.5, top_k=-1):
+        """Non-maximum suppression.
+        Arguments:
+            box_scores: a float numpy array of shape [n, 5],
+                where each row is (xmin, ymin, xmax, ymax, score).
+            overlap_threshold: a float number.
+        Returns:
+            list with indices of the selected boxes
+        """
+        # 如果沒有有效的候選區域則回傳空的清單
+        box_scores = to_tensor(box_scores)
+        if len(box_scores) == 0:
+            return []
+        score = box_scores[:, 4]
+        boxes = box_scores[:, :4]
+        # 存放過關的索引值
+        picked = []
+        # 依照機率信心水準升冪排序
+        indexes = argsort(score, descending=False)
+
+        while len(indexes) > 0:
+            # 如此一來，最後一筆即是信心水準最高值
+            # 加入至過關清單中
+            current = indexes[-1]
+            picked.append(current.item())
+
+            # 計算其餘所有候選框與此當前框之間的IOU
+
+            if 0 < top_k == len(picked) or len(indexes) == 1:
+                break
+            current_box = boxes[current, :]
+            current_score = score[current]
+            # 除了最後一筆以外的都是其餘框
+            indexes = indexes[:-1]
+            rest_boxes = boxes[indexes, :]
+            iou = self.iou_of(
+                rest_boxes,
+                expand_dims(current_box, axis=0),
+            )
+            # IOU未超過門檻值的表示未與當前框重疊，則留下，其他排除
+            indexes = indexes[iou <= overlap_threshold]
+        return box_scores[picked]
 
 
     def infer_single_image(self,img,**kwargs):
@@ -327,7 +412,8 @@ class Mtcnn(ImageDetectionModel):
             boxes_list=[]
             for i in range(len(scales)):
                 scaled_img=imgs[i]
-                inp = to_tensor(np.expand_dims(scaled_img, 0)).to(torch.device("cuda" if self.pnet.weights[0].data.is_cuda else "cpu")).to(self.pnet.weights[0].data.dtype)
+                inp =to_tensor(expand_dims(scaled_img, 0)).to(torch.device("cuda" if self.pnet.weights[0].data.is_cuda else "cpu")).to(self.pnet.weights[0].data.dtype)
+
                 boxes=self.pnet(inp)
                 if boxes is not None and len(boxes)>0:
                     scale=scales[i]
@@ -345,8 +431,9 @@ class Mtcnn(ImageDetectionModel):
 
                 #print('total {0} boxes in pnet in all scale '.format(len(boxes)))
                 boxes=clip_boxes_to_image(boxes,(img.shape[0],img.shape[1]))
-                boxes =nms(boxes, threshold=self.detection_threshould[0])
-                print('pnet:{0} boxes '.format(len(boxes)))
+                boxes =self.boxes_nms(boxes, overlap_threshold=self.detection_threshould[0])
+                if self.verbose:
+                    print('pnet:{0} boxes '.format(len(boxes)))
                 #print('total {0} boxes after nms '.format(len(boxes)))
                 #score = to_numpy(boxes[:, 4]).reshape(-1)
                 if boxes is not None:
@@ -359,7 +446,7 @@ class Mtcnn(ImageDetectionModel):
                         box = boxes[k]
                         crop_img = img.copy()[int(box[1]):int(box[3]), int(box[0]):int(box[2]), :]
                         if crop_img.shape[0] > 0 and crop_img.shape[1] > 0:
-                            new_arr[k] = resize((24, 24))(crop_img / 255.0).transpose([2, 0, 1])
+                            new_arr[k] = resize((24, 24))(crop_img).transpose([2, 0, 1]) / 255.0
                         # else:
                         #     print(box)
                     new_arr = to_tensor(new_arr)
@@ -379,10 +466,13 @@ class Mtcnn(ImageDetectionModel):
                     else:
                         r_out1, r_out2, r_out3 = self.rnet(new_arr)
 
-                    probs = to_numpy(r_out1)
-                    keep = np.where(probs[:, 0] > self.detection_threshould[1])[0]
+                    probs =r_out1
+                    keep =probs[:, 0] > self.detection_threshould[1]
                     r_out1=r_out1[keep]
+
                     boxes = boxes[keep]
+                    if len(boxes)==0:
+                        return boxes
                     boxes[:, 4] = r_out1[:, 0]
                     r_out2 = r_out2[keep]
                     boxes=calibrate_box(boxes,r_out2)
@@ -391,12 +481,12 @@ class Mtcnn(ImageDetectionModel):
                     #######################################
                     #########rnet finish
                     #######################################
-
-                    boxes=nms(boxes,  threshold=self.detection_threshould[1],image_size=(img.shape[0],img.shape[1]),min_size=self.min_size)
-                    print('rnet:{0} boxes '.format(len(boxes)))
+                    boxes=self.boxes_nms(boxes, overlap_threshold=self.detection_threshould[1])
+                    if self.verbose:
+                        print('rnet:{0} boxes '.format(len(boxes)))
                     #print('total {0} boxes after nms '.format(len(boxes)))
                     boxes = clip_boxes_to_image(boxes, (img.shape[0], img.shape[1]))
-                    boxes=self.rerec(boxes,img.shape)
+                    boxes=self.rerec(to_tensor(boxes),img.shape)
                     new_arr=np.zeros((boxes.shape[0],3,48,48))
 
 
@@ -404,17 +494,18 @@ class Mtcnn(ImageDetectionModel):
                         box=boxes[k]
                         crop_img=img.copy()[int(box[1]):int(box[3]),int(box[0]):int(box[2]),:]
                         if crop_img.shape[0]>0 and crop_img.shape[1]>0:
-                            new_arr[k]=resize((48,48))(crop_img/255.0).transpose([2,0,1])
+                            new_arr[k]=resize((48,48))(crop_img).transpose([2,0,1])/255.0
                         # else:
                         #     print(box)
 
                     new_arr=to_tensor(new_arr)
                     o_out1, o_out2,o_out3  = self.onet(new_arr)
-                    probs = to_numpy(o_out1)
-                    keep = np.where(probs[:, 0] > self.detection_threshould[2])[0]
+                    probs = o_out1
+                    keep = probs[:, 0] > self.detection_threshould[2]
                     o_out1 = o_out1[keep]
                     boxes = boxes[keep]
-
+                    if len(boxes)==0:
+                        return boxes
                     boxes[:, 4] = o_out1[:, 0]
                     o_out2 = o_out2[keep]
                     o_out3=o_out3[keep]
@@ -429,9 +520,10 @@ class Mtcnn(ImageDetectionModel):
                     #######################################
                     #########onet finish
                     #######################################
-                    boxes=nms(boxes, threshold=self.detection_threshould[2],image_size=(img.shape[0],img.shape[1]),min_size=self.min_size)
-                    print('onet:{0} boxes '.format(len(boxes)))
-                    return boxes
+                    boxes = self.boxes_nms(boxes, overlap_threshold=self.detection_threshould[2])
+                    if self.verbose:
+                        print('onet:{0} boxes '.format(len(boxes)))
+                    return to_numpy(boxes)
             else:
                 return None
             #idx=int(np.argmax(result,-1)[0])
