@@ -20,16 +20,16 @@ from torch._six import container_abcs
 from torch.nn import init
 from torch.nn.parameter import Parameter
 
-from ..backend.common import *
-from ..backend.pytorch_backend import to_numpy, to_tensor, Layer, Sequential
-from ..data.image_common import *
-from ..data.utils import download_model_from_google_drive
-from ..layers.pytorch_activations import get_activation, Identity
-from ..layers.pytorch_blocks import *
-from ..layers.pytorch_layers import *
-from ..layers.pytorch_normalizations import get_normalization
-from ..layers.pytorch_pooling import *
-from ..optims.pytorch_trainer import *
+from trident.backend.common import *
+from trident.backend.pytorch_backend import to_numpy, to_tensor, Layer, Sequential
+from trident.data.image_common import *
+from trident.data.utils import download_model_from_google_drive
+from trident.layers.pytorch_activations import get_activation, Identity
+from trident.layers.pytorch_blocks import *
+from trident.layers.pytorch_layers import *
+from trident.layers.pytorch_normalizations import get_normalization
+from trident.layers.pytorch_pooling import *
+from trident.optims.pytorch_trainer import *
 
 __all__ = ['basic_block','bottleneck', 'ResNet','ResNet50','ResNet101','ResNet152']
 
@@ -84,66 +84,34 @@ def bottleneck(num_filters=64,strides=1,expansion = 4,conv_shortcut=True,use_bia
 #         model.load_state_dict(state_dict)
 #     return model
 
-def ResNet(block, layers, input_shape=(3, 224, 224), num_classes=1000, use_bias=False, zero_init_residual=False,
-           width_per_group=64, replace_stride_with_dilation=None, include_top=True, model_name='',
+def ResNet(block, layers, input_shape=(3, 224, 224), num_classes=1000, use_bias=False,  include_top=True, model_name='',
            **kwargs):
     """Instantiates the ResNet, ResNetV2, and ResNeXt architecture.
-    Optionally loads weights pre-trained on ImageNet.
-    Note that the data format convention used by the model is
-    the one specified in your Keras config at `~/.keras/keras.json`.
-    # Arguments
-        stack_fn: a function that returns output tensor for the
-            stacked residual blocks.
-        preact: whether to use pre-activation or not
-            (True for ResNetV2, False for ResNet and ResNeXt).
-        use_bias: whether to use biases for convolutional layers or not
-            (True for ResNet and ResNetV2, False for ResNeXt).
-        model_name: string, model name.
-        include_top: whether to include the fully-connected
-            layer at the top of the network.
-        weights: one of `None` (random initialization),
-              'imagenet' (pre-training on ImageNet),
-              or the path to the weights file to be loaded.
-        input_tensor: optional Keras tensor
-            (i.e. output of `layers.Input()`)
-            to use as image input for the model.
+
+    Args
+        block: a function that returns output tensor for the stacked residual blocks.
+        layers: list of integer, the number of  repeat units in each blocks.
         input_shape: optional shape tuple, only to be specified
             if `include_top` is False (otherwise the input shape
-            has to be `(224, 224, 3)` (with `channels_last` data format)
-            or `(3, 224, 224)` (with `channels_first` data format).
+            has to be`(3, 224, 224)` (with `channels_first` data format).
             It should have exactly 3 inputs channels.
-        pooling: optional pooling mode for feature extraction
-            when `include_top` is `False`.
-            - `None` means that the output of the model will be
-                the 4D tensor output of the
-                last convolutional layer.
-            - `avg` means that global average pooling
-                will be applied to the output of the
-                last convolutional layer, and thus
-                the output of the model will be a 2D tensor.
-            - `max` means that global max pooling will
-                be applied.
-        classes: optional number of classes to classify images
+        num_classes: optional number of classes to classify images
             into, only to be specified if `include_top` is True, and
             if no `weights` argument is specified.
-    # Returns
+        use_bias: whether to use biases for convolutional layers or not
+            (True for ResNet and ResNetV2, False for ResNeXt).
+        include_top: whether to include the fully-connected layer at the top of the network.
+        model_name: string, model name.
+
+    Returns
         A Keras model instance.
-    # Raises
-        ValueError: in case of invalid argument for `weights`,
-            or invalid input shape.
+
+    Raises
+        ValueError: in case of invalid argument for `weights`,  or invalid input shape.
+
     """
 
 
-    # if not (weights in {'imagenet', None} or os.path.exists(weights)):
-    #     raise ValueError('The `weights` argument should be either '
-    #                      '`None` (random initialization), `imagenet` '
-    #                      '(pre-training on ImageNet), '
-    #                      'or the path to the weights file to be loaded.')
-    #
-    # if weights == 'imagenet' and include_top and classes != 1000:
-    #     raise ValueError('If using `weights` as `"imagenet"` with `include_top`'
-    #                      ' as true, `classes` should be 1000')
-
     def _make_layer(block, num_filters, blocklayers, strides=1, dilate=False,use_bias=use_bias,layer_name=''):
         conv_shortcut=False
         if strides!=1 or block is bottleneck:
@@ -173,7 +141,7 @@ def ResNet(block, layers, input_shape=(3, 224, 224), num_classes=1000, use_bias=
         resnet.add_module('softmax', SoftMax(name='softmax'))
     resnet.name=model_name
     model=ImageClassificationModel(input_shape=input_shape,output=resnet)
-    model.signature = get_signature(model.model.forward)
+
 
     with open(os.path.join(os.path.dirname(os.path.abspath(__file__)) ,'imagenet_labels1.txt'), 'r', encoding='utf-8-sig') as f:
         labels = [l.rstrip() for l in f]
@@ -183,16 +151,33 @@ def ResNet(block, layers, input_shape=(3, 224, 224), num_classes=1000, use_bias=
     return model
 
 #
-# def ResNet18(include_top=True,
-#              weights='imagenet',
-#              input_shape=None,
-#              classes=1000,
-#              **kwargs):
-#     if input_shape is not None and len(input_shape)==3:
-#         input_shape=tuple(input_shape)
-#     else:
-#         input_shape=(3, 224, 224)
-#     resnet18 = ResNet(basic_block, [2, 2, 2, 2], input_shape, model_name='resnet18')
+def ResNet18(include_top=True,
+             pretrained=True,
+             input_shape=None,
+             classes=1000,
+             **kwargs):
+    if input_shape is not None and len(input_shape)==3:
+        input_shape=tuple(input_shape)
+    else:
+        input_shape=(3, 224, 224)
+    resnet18 = ResNet(basic_block, [2, 2, 2, 2], input_shape, model_name='resnet18')
+    if pretrained == True:
+        download_model_from_google_drive(model_urls['resnet18'], dirname, 'resnet18.pth')
+        recovery_model = torch.load(os.path.join(dirname, 'resnet18.pth'))
+        recovery_model.name = 'resnet18'
+        recovery_model.eval()
+        recovery_model.to(_device)
+        if include_top == False:
+            recovery_model.__delitem__(-1)
+            recovery_model.__delitem__(-1)
+            recovery_model.__delitem__(-1)
+        else:
+            if classes != 1000:
+                resnet18.class_names = []
+                recovery_model.fc = Dense(classes, activation=None, name='fc')
+                recovery_model.fc.input_shape = recovery_model.avg_pool.output_shape
+        resnet18.model = recovery_model
+    return resnet18
 
 def ResNet50(include_top=True,
              pretrained=True,
@@ -207,21 +192,21 @@ def ResNet50(include_top=True,
     if pretrained==True:
         download_model_from_google_drive(model_urls['resnet50'],dirname,'resnet50.pth')
         recovery_model=torch.load(os.path.join(dirname,'resnet50.pth'))
+        recovery_model.name = 'resnet50'
         recovery_model.eval()
         recovery_model.to(_device)
         if include_top==False:
             recovery_model.__delitem__(-1)
+            recovery_model.__delitem__(-1)
+            recovery_model.__delitem__(-1)
+            resnet50.class_names = []
         else:
             if classes!=1000:
-
-                new_fc = Dense(classes, activation=None, name='fc')
-                new_fc.input_shape=recovery_model.fc.input_shape
-                recovery_model.fc=new_fc
-
+                resnet50.class_names = []
+                recovery_model.fc= Dense(classes, activation=None, name='fc')
+                recovery_model.fc.input_shape=recovery_model.avg_pool.output_shape
 
         resnet50.model=recovery_model
-        resnet50.rebinding_input_output(input_shape)
-        resnet50.signature = get_signature(resnet50.model.forward)
     return resnet50
 
 def ResNet101(include_top=True,
@@ -237,17 +222,21 @@ def ResNet101(include_top=True,
     if pretrained==True:
         download_model_from_google_drive(model_urls['resnet101'],dirname,'resnet101.pth')
         recovery_model=torch.load(os.path.join(dirname,'resnet101.pth'))
+        recovery_model.name = 'resnet101'
         recovery_model.eval()
         recovery_model.to(_device)
         if include_top == False:
             recovery_model.__delitem__(-1)
+            recovery_model.__delitem__(-1)
+            recovery_model.__delitem__(-1)
+            resnet101.class_names = []
         else:
             if classes != 1000:
+                resnet101.class_names = []
                 recovery_model.fc = Dense(classes, activation=None, name='fc')
+                recovery_model.fc.input_shape=recovery_model.avg_pool.output_shape
 
         resnet101.model=recovery_model
-        resnet101.rebinding_input_output(input_shape)
-        resnet101.signature = get_signature(resnet101.model.forward)
     return resnet101
 
 
@@ -264,17 +253,21 @@ def ResNet152(include_top=True,
     if pretrained==True:
         download_model_from_google_drive(model_urls['resnet152'],dirname,'resnet152.pth')
         recovery_model=torch.load(os.path.join(dirname,'resnet152.pth'))
+        recovery_model.name = 'resnet152'
         recovery_model.eval()
         recovery_model.to(_device)
         if include_top == False:
             recovery_model.__delitem__(-1)
+            recovery_model.__delitem__(-1)
+            recovery_model.__delitem__(-1)
+            resnet152.class_names = []
         else:
             if classes != 1000:
+                resnet152.class_names=[]
                 recovery_model.fc = Dense(classes, activation=None, name='fc')
+                recovery_model.fc.input_shape=recovery_model.avg_pool.output_shape
 
         resnet152.model=recovery_model
-        resnet152.rebinding_input_output(input_shape)
-        resnet152.signature = get_signature(resnet152.model.forward)
     return resnet152
 
 
